@@ -23,7 +23,6 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
-/*#include <stdio.h>*/
 
 #ifdef HAVE_LOCALE
 #ifdef HAVE_LANGINFO_CODESET
@@ -42,18 +41,18 @@ char * char_conv_from = NULL;
 #endif
 
 #define BUFFER_SIZE	1024
+static void closeCharSetConversion();
 
-int setCharSetConversion(char * to, char * from) {
+static int setCharSetConversion(char * to, char * from) {
 #ifdef HAVE_ICONV
 	if(char_conv_to && strcmp(to,char_conv_to)==0 &&
 			char_conv_from && strcmp(from,char_conv_from)==0)
-	{ 
 		return 0;
-	}
 
 	closeCharSetConversion();
 
-	if((char_conv_iconv = iconv_open(to,from))==(iconv_t)(-1)) return -1;
+	if ((char_conv_iconv = iconv_open(to,from))==(iconv_t)(-1))
+		return -1;
 
 	char_conv_to = strdup(to);
 	char_conv_from = strdup(from);
@@ -63,7 +62,7 @@ int setCharSetConversion(char * to, char * from) {
 	return -1;
 }
 
-char * convStrDup(char * string) {
+static char * convStrDup(char * string) {
 #ifdef HAVE_ICONV
 	char buffer[BUFFER_SIZE];
 	size_t inleft = strlen(string);
@@ -98,7 +97,7 @@ char * convStrDup(char * string) {
 	return NULL;
 }
 
-void closeCharSetConversion() {
+static void closeCharSetConversion(void) {
 #ifdef HAVE_ICONV
 	if(char_conv_to) {
 		iconv_close(char_conv_iconv);
@@ -110,7 +109,7 @@ void closeCharSetConversion() {
 #endif
 }
 
-void setLocaleCharset() {
+void setLocaleCharset(void) {
 #ifdef HAVE_LOCALE
 #ifdef HAVE_LANGINFO_CODESET
         char * originalLocale;
