@@ -38,18 +38,14 @@
 #define SIMPLE_CMD(funcname, libmpdclient_funcname, ret) \
 int funcname ( int argc, char ** argv, mpd_Connection * conn) { \
         libmpdclient_funcname(conn); \
-	printErrorAndExit(conn); \
         my_finishCommand(conn); \
-	printErrorAndExit(conn); \
         return ret; \
 }
 
 #define SIMPLE_ONEARG_CMD(funcname, libmpdclient_funcname, ret) \
 int funcname ( int argc, char ** argv, mpd_Connection * conn) { \
         libmpdclient_funcname(conn, toUtf8(argv[0])); \
-	printErrorAndExit(conn); \
         my_finishCommand(conn); \
-	printErrorAndExit(conn); \
         return ret; \
 }
 
@@ -132,6 +128,11 @@ int cmd_add (int argc, char ** argv, mpd_Connection * conn )
 			while((sp = strchr(sp,' '))) *sp = '_';
 			len = strlen(dup);
 			for(i=0;i<argc;i++) {
+				if (strcmp(argv[i], "/") == 0) {
+					insertInListWithoutKey(lists[i],
+					                       strdup(fromUtf8(song->file)));
+					continue;
+				}
 				if(len<arglens[i]) continue;
 				ret = strncmp(argv[i],dup,
 						arglens[i]);
@@ -183,9 +184,7 @@ int cmd_add (int argc, char ** argv, mpd_Connection * conn )
 int cmd_crop( int argc, char ** argv, mpd_Connection * conn )
 {
 
-	mpd_Status *status;
-	status = getStatus( conn );
-
+	mpd_Status *status = getStatus( conn );
 	int length = ( status->playlistLength - 1 );
 
 	if( status->playlistLength == 0 ) {
@@ -288,9 +287,9 @@ int cmd_toggle( int argc, char ** argv, mpd_Connection * conn )
 	status = getStatus(conn);
 
 	if(status->state==MPD_STATUS_STATE_PLAY) {
-		cmd_pause(0,0,conn);
+		cmd_pause(0, NULL, conn);
 	} else {
-		cmd_play(0,0,conn);
+		cmd_play(0, NULL, conn);
 	}
 	return 1;
 }
@@ -334,7 +333,7 @@ int cmd_enable( int argc, char ** argv, mpd_Connection * conn )
 	}
 
 	mpd_finishCommand( conn );
-	cmd_outputs( 0, 0, conn );
+	cmd_outputs( 0, NULL, conn );
 	return( 0 );
 }
 
@@ -349,7 +348,7 @@ int cmd_disable( int argc, char ** argv, mpd_Connection * conn )
 	}
 	
 	mpd_finishCommand( conn );
-	cmd_outputs( 0, 0, conn );
+	cmd_outputs( 0, NULL, conn );
 	return( 0 );
 }
 
@@ -380,7 +379,6 @@ int cmd_seek ( int argc, char ** argv, mpd_Connection * conn )
 	mpd_Status * status;
 	char * arg = argv[0];
 	char * test;
-	char * last_char;
 
 	int seekchange;
 	int total_secs;
@@ -392,15 +390,14 @@ int cmd_seek ( int argc, char ** argv, mpd_Connection * conn )
 	if(status->state==MPD_STATUS_STATE_STOP)
 		DIE("not currently playing\n");
 
-        if(*arg == '+')
-                rel = 1;
-        else if(*arg == '-')
-                rel = -1;
+	/* Detect +/- if exists point to the next char */
+        if(*arg == '+') rel = 1;
+        else if(*arg == '-') rel = -1;
 
-	last_char = &arg[strlen(arg)-1];
+	if(rel != 0) arg++;
 
 	/* If seeking by percent */
-	if( *last_char == '%' ) {
+	if( arg[strlen(arg)-1] == '%' ) {
 
 		double perc;
 
@@ -488,10 +485,17 @@ int cmd_seek ( int argc, char ** argv, mpd_Connection * conn )
 		seekchange = total_secs;
 	}
 
-	seekto = (rel ? status->elapsedTime + seekchange : seekchange);
+	/* This detects +/- and is necessary due to the parsing of HH:MM:SS numbers*/
+	if(rel == 1) {
+		seekto = status->elapsedTime + seekchange;
+	} else if (rel == -1) {
+		seekto = status->elapsedTime - seekchange;
+	} else {
+		seekto = seekchange;
+	}
 
 	if(seekto > status->totalTime)
-		DIE("seek amount would seek past the end of the song\n");
+		DIE("Seek amount would seek past the end of the song\n");
 
 	mpd_sendSeekIdCommand(conn,status->songid,seekto);
 	printErrorAndExit(conn);
@@ -738,7 +742,7 @@ int cmd_search ( int argc, char ** argv, mpd_Connection * conn )
 		{"artist", MPD_TABLE_ARTIST },
 		{"title", MPD_TABLE_TITLE },
 		{"filename", MPD_TABLE_FILENAME },
-		{}
+		{ NULL }
 	};
 
 	for(i=0;mpc_search[i].field;++i)
@@ -1038,5 +1042,6 @@ int cmd_stats ( int argc, char ** argv, mpd_Connection * conn )
 
 int cmd_status ( int argc, char ** argv, mpd_Connection * conn )
 {
-	return 1;
+	print_status(conn);
+	return 0;
 }
