@@ -35,9 +35,9 @@
 #include <sys/param.h>
 
 struct _mpc_table {
-	const char * command; 
+	const char * command;
 	const int min, max;	/* min/max arguments allowed, -1 = unlimited */
-	int pipe;		/* 
+	int pipe;		/*
 				1: implicit pipe read, `-' optional as argv[2]
 				2: explicit pipe read `-' needed as argv[2]
 				3: explicit pipe read `-' needed as argv[3]
@@ -49,10 +49,10 @@ struct _mpc_table {
 	char * help;		/* NULL means they won't be shown in help */
 } mpc_table [] = {
 	/* Command Name, Minimum Arguments, Maximium args, pipe read (see above), Argument description, command description */
-	{"add", 0, -1, 1, cmd_add,"<filename>","Add a song to the current playlist" },
+	{"add", 0, -1, 1, cmd_add,"<file>","Add a song to the current playlist" },
 	{"crop", 0, 0, 0, cmd_crop,"","Remove all songs except for the currently playing song" },
-	{"del", 0, -1, 1, cmd_del,"<playlist #>","Remove a song from the current playlist" },
-	{"play", 0, -1, 2, cmd_play,"<number>","Start playing at <number> (default: 1)" },
+	{"del", 0, -1, 1, cmd_del,"<position>","Remove a song from the current playlist" },
+	{"play", 0, -1, 2, cmd_play,"[<position>]","Start playing at <position> (default: 1)" },
 	{"next", 0, 0, 0, cmd_next, "","Play the next song in the current playlist"},
 	{"prev", 0, 0, 0, cmd_prev,"","Play the previous song in the current playlist"},
 	{"pause", 0, 0, 0, cmd_pause, "", "Pauses the currently playing song"},
@@ -60,16 +60,14 @@ struct _mpc_table {
 	{"stop", 0, 0, 0, cmd_stop,"", "Stop the currently playing playlists"},
 	{"seek", 1, 1, 0, cmd_seek,"[+-][HH:MM:SS] or <0-100>%","Seeks to the specified position"},
 	{"clear", 0, 0, 0, cmd_clear,"", "Clear the current playlist"},
-	/*
 	{"outputs", 0, 0, 0, cmd_outputs,"", "Show the current outputs"},
 	{"enable", 1, 1, 0, cmd_enable, "<output #>", "Enable a output"},
 	{"disable", 1, 1, 0, cmd_disable, "<output #>", "Disable a output"},
-	*/
 	{"shuffle", 0, 0, 0, cmd_shuffle,"", "Shuffle the current playlist"},
 	{"move", 2, 2, 0, cmd_move,"<from> <to>", "Move song in playlist"},
 	{"playlist", 0, 0, 0, cmd_playlist, "", "Print the current playlist"},
-	{"listall", 0, -1, 2, cmd_listall,"[<song>]","List all songs in the music dir"},
-	{"ls", 0, -1, 2, cmd_ls,"[<dir>]","List the contents of <dir>"},
+	{"listall", 0, -1, 2, cmd_listall,"[<file>]","List all songs in the music dir"},
+	{"ls", 0, -1, 2, cmd_ls,"[<directory>]","List the contents of <directory>"},
 	{"lsplaylists", 0, -1, 2, cmd_lsplaylists,"", "Lists currently available playlists"},
 	{"load", 0, -1, 1, cmd_load,"<file>","Load <file> as a playlist"},
 	{"save", 1, 1, 0, cmd_save,"<file>", "Saves a playlist as <file>"},
@@ -78,47 +76,65 @@ struct _mpc_table {
 	{"repeat", 0, 1, 0, cmd_repeat,"<on|off>", "Toggle repeat mode, or specify state"},
 	{"random", 0, 1, 0, cmd_random,"<on|off>", "Toggle random mode, or specify state"},
 	{"search", 2, -1, 3, cmd_search, "<type> <queries>",	"Search for a song"},
-	{"crossfade", 0, 1, 0, cmd_crossfade,"[sec]","Set and display crossfade settings"},
-	{"update", 0, -1, 2, cmd_update,	"", "Scans music directory for updates"},
-	{"stats", 0, -1, 0, cmd_stats, "", "Displays statistics about MPD"}, 
+	{"crossfade", 0, 1, 0, cmd_crossfade,"[<seconds>]","Set and display crossfade settings"},
+	{"update", 0, -1, 2, cmd_update, "[<path>]", "Scans music directory for updates"},
+	{"stats", 0, -1, 0, cmd_stats, "", "Displays statistics about MPD"},
 	{"version", 0, 0, 0, cmd_version,"", "Reports version of MPD"},
-	/*  loadtab, lstab, and tab used for completion-scripting only */
-	{"loadtab",0, 1, 0, cmd_loadtab,"<directory>",}, 
-	{"lstab",0, 1, 0, cmd_lstab,"<directory>",}, 
-	{"tab",0, 1, 0, cmd_tab,"<directory/file>",},
+	/* loadtab, lstab, and tab used for completion-scripting only */
+	{"loadtab",0, 1, 0, cmd_loadtab,"<directory>",},
+	{"lstab",0, 1, 0, cmd_lstab,"<directory>",},
+	{"tab",0, 1, 0, cmd_tab,"<path>",},
 	/* status was added for pedantic reasons */
-	{"status", 0, -1, 0, cmd_status, "", NULL }, 
+	{"status", 0, -1, 0, cmd_status, "", NULL },
 	/* don't remove this, when mpc_table[i].command is NULL it will terminate the loop */
-	{}
+	{ NULL }
 };
 
-void print_help (int max, char * progname, char * command)
+static int print_help(char * progname, char * command)
 {
-	int i;
+	int i, max = 0;
+	int ret = EXIT_FAILURE;
+	FILE *outfp = stderr;
 
-	if(command && strcmp(command,"help"))
-			fprintf(stderr,"unknown command \"%s\"\n",command);
-	fprintf(stderr,"Usage: %s <command> [command args]...\n"
+	if (command) {
+		if (!strcmp(command,"help")) {
+			outfp = stdout;
+			ret = EXIT_SUCCESS;
+		} else
+			fprintf(outfp,"unknown command \"%s\"\n",command);
+	}
+	fprintf(outfp,"Usage: %s <command> [command args]...\n"
 		"mpc version: "VERSION"\n",progname);
-	fprintf(stderr,	"%s %*s  Displays status\n",progname,max," ");
-	
+
+	for (i=0; mpc_table[i].command; ++i) {
+		if (mpc_table[i].help) {
+			int tmp = strlen(mpc_table[i].command) +
+					strlen(mpc_table[i].usage);
+			max = (tmp > max) ? tmp : max;
+		}
+	}
+
+	fprintf(outfp,	"%s %*s  Displays status\n",progname,max," ");
+
 	for (i=0; mpc_table[i].command; ++i) {
 		int spaces;
-		
+
 		if (!mpc_table[i].help)
 			continue ;
 		spaces = max-(strlen(mpc_table[i].command)+strlen(mpc_table[i].usage));
 		spaces += !spaces ? 0 : 1;
 
-		fprintf(stderr,"%s %s %s%*s%s\n",progname,
+		fprintf(outfp,"%s %s %s%*s%s\n",progname,
 			mpc_table[i].command,mpc_table[i].usage,
 			spaces," ",mpc_table[i].help);
-		
+
 	}
-	fprintf(stderr,"For more information about these and other options look at man 1 mpc\n");
+	fprintf(outfp,"For more information about these and other "
+			"options look at man 1 mpc\n");
+	return ret;
 }
 
-mpd_Connection * setup_connection ()
+static mpd_Connection * setup_connection (void)
 {
 	char * host = DEFAULT_HOST;
 	char * port = DEFAULT_PORT;
@@ -129,7 +145,7 @@ mpd_Connection * setup_connection ()
 	int password_len= 0;
 	int parsed_len = 0;
 	mpd_Connection * conn;
-	
+
 	if((test = getenv("MPD_HOST"))) {
 		host =test;
 		host_env = 1;
@@ -141,7 +157,7 @@ mpd_Connection * setup_connection ()
 	}
 
 	iport = strtol(port,&test,10);
-	
+
 	if(iport<0 || *test!='\0') {
 		fprintf(stderr,"MPD_PORT \"%s\" is not a positive integer\n",
 				port);
@@ -149,36 +165,35 @@ mpd_Connection * setup_connection ()
 	}
 
 	parse_password(host, &password_len, &parsed_len);
-	
+
 	conn = mpd_newConnection(host+parsed_len,iport,10);
 
-	if(conn->error && (!port_env || !host_env)) 
+	if(conn->error && (!port_env || !host_env))
 		fprintf(stderr,"MPD_HOST and/or MPD_PORT environment variables"
 			" are not set\n");
-	
+
 	printErrorAndExit(conn);
 
-	if(password_len) 
+	if(password_len)
 		send_password (host, password_len, conn);
 
 	return conn;
 }
 
-void print_status_and_exit () 
+static int print_status_and_exit (void)
 {
 	mpd_Connection * conn = setup_connection();
 	print_status(conn);
 	mpd_closeConnection(conn);
-	fclose(stdout);
-	exit(EXIT_SUCCESS);
+	return EXIT_SUCCESS;
 }
 
 /* check arguments to see if they are valid */
-char ** check_args (int idx, int * argc, char ** argv)
+static char ** check_args(int idx, int * argc, char ** argv)
 {
 	char ** array;
 	int i;
-	
+
 	if ( ( mpc_table[idx].pipe==1 &&
 		(2==*argc || (3==*argc && 0==strcmp(argv[2],STDIN_SYMBOL) )))
 	|| (mpc_table[idx].pipe==2 && (3==*argc && 0==strcmp(argv[2],STDIN_SYMBOL)))
@@ -204,42 +219,37 @@ char ** check_args (int idx, int * argc, char ** argv)
 
 int main(int argc, char ** argv)
 {
-	int i = 0;
-	int helplen = 0;
-	int ret = 0;
+	int i, ret;
+	const char *cmd;
 	setLocaleCharset();
 
-	if(parse_options(&argc, argv) < 0) {
-		print_help(helplen,argv[0],NULL);
-		exit(EXIT_FAILURE);
-	}
+	if(parse_options(&argc, argv) < 0)
+		return print_help(argv[0],NULL);
 
 	if (argc==1)
-		print_status_and_exit ();
+		return print_status_and_exit();
 
+	cmd = argv[1];
 	for (i = 0; mpc_table[i].command; ++i) {
-		if ( ! strcmp(argv[1], mpc_table[i].command) ) {
+		if ( ! strcmp(cmd, mpc_table[i].command) ) {
 			char ** array = check_args (i, &argc , argv);
 			mpd_Connection * conn = setup_connection();
-			
+
 			/* not a typo, assignment intended */
-			if( (ret=mpc_table[i].handler(argc, array, conn)))
-				print_status(conn);
-			
+			if( (ret=mpc_table[i].handler(argc, array, conn))) {
+				struct mpc_option* nostatus = get_option ("no-status");
+				if (! nostatus->set)
+					print_status(conn);
+			}
+
 			if (0>mpc_table[i].pipe)
 				free_pipe_array(argc,array);
-			
+
 			free(array);
 			mpd_closeConnection(conn);
-			fclose(stdout);
-			return EXIT_SUCCESS;
-		} else if (mpc_table[i].help) {
-			int tmp = strlen(mpc_table[i].command) + strlen(mpc_table[i].usage);
-			helplen = (tmp > helplen) ? tmp : helplen;
+			return (ret >= 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 		}
 	}
-	print_help(helplen,argv[0],argv[1]);
-	fclose(stdout);
-	return EXIT_SUCCESS;
+	return print_help(argv[0],argv[1]);
 }
 
