@@ -84,7 +84,7 @@ int get_boolean (const char * arg)
 		{ "1", "0" },
 		{ "true", "false" },
 		{ "yes", "no" },
-		{}
+		{ NULL }
 	};
 
 	for (i=0; bool_table[i].on; ++i) {
